function [nsp,nerr,nerr_ch,nblk,nblk_ch,R]=std_det_4a(det_type,order,modulation,levels,Esn0,channel,Nbit,Ntot, ...
                                                      Nblkerrmax,pulse,Tc,dt,Nlev,Nu,pnseq,asynch,cc, ...
                                                      soft_decode,pnseq_bin,Nguard,Nint_ed)
%              
%function [nsp,nerr,nerr_ch,nblk,nblk_ch,R]=std_det_4a(det_type,order,modulation,levels,Esn0,channel,Nbit,Ntot, ...
%                                                      Nblkerrmax,pulse,Tc,dt,Nlev,Nu,pnseq,asynch,cc, ...
%                                                      soft_decode,pnseq_bin,Nguard,Nint_ed)
%
% Detection for UWB-IR
%
% INPUT PARAMETERS:
% det_type = no use here
% order = RAKE order
% modulation = 3 --> PPM + PAM
% levels = modulation levels (PPM)
% channel = 0 --> ideal AWGN, >0 --> IEEE 802.15.4a + AWGN 
% Esn0 (dB) = Es/N0, Es = energy per symbol
% Nbit = number of bits per block (uncoded) 
% Ntot = max number of blocks to be simulated
% Nblkerrmax = max number of wrong blocks before ending simulation 
% pulse = pulse waveform
% Tc = chip time
% dt = time resolution
% Nlev = number of levels in the frame
% Nu = number of active users
% pnseq = Pseudo Noise level sequence
% asynch = 0 -> synchronous users, 1 -> asynchronous users
% cc --> convolutional code (from poly2trellis.m)
% soft_decode --> 0 -> Viterbi Hard decoding, 1 -> Viterbi soft decoding, 2 -> both
% pnseq_bin = representation of Pseudo Noise level sequence at dt level
% Nguard = guard chip times x frame
% Nint_ed = N. chip time for non-coherent integration
%
% OUTPUT PARAMETERS:
% nsp = number of simulated blocks at each Es/N0
% nerr = number of incorrect bits at each Es/N0 (column) and each configuration (row)
% nerr_ch = number of incorrect bits (uncoded) at each Es/N0 (column) and each configuration (row)
% nblk = number of incorrect blocks at each Es/N0 (column) and each configuration (row)
% nblk_ch = number of incorrect blocks (uncoded) at each Es/N0 (column) and each configuration (row)
% [Results are stored in the matrices nerr,nerr_ch,nblk,nblk_ch in the following order: 
%  Coherent - Selective RAKE - PPM, Non-coherent - Selective RAKE - PPM,
%  Non-coherent PPM, Coherent - Selective RAKE - BPSK
%  Note: if soft_decode == 2 --> odd rows = soft decoding, even rows = hard decoding
% R = transmission rate
%
% Written by : Luca Reggiani, Politecnico di Milano, Italy
% Version    : 15/07/2005 


% force modulation to the format PAM + PPM
modulation=3;

if (nargin==13)
    Nu=1;
    pnseq=[];
    asynch=0;
    cc=[];
    soft_decode=0;
    pnseq_bin=[];
    Nguard=[0];
    Nint_ed=1;
end
if (nargin==14)
    pnseq=[];
    asynch=0;
    cc=[];
    soft_decode=0;
    pnseq_bin=[];
    Nguard=[0];
    Nint_ed=1;
end
if (nargin==15)
    asynch=0;
    cc=[];
    soft_decode=0;
    pnseq_bin=[];
    Nguard=[0];
    Nint_ed=1;
end
if (nargin==16)
    cc=[];
    soft_decode=0;
    pnseq_bin=[];
    Nguard=[0];
    Nint_ed=1;
end
if (nargin==17)
    soft_decode=0;
    pnseq_bin=[];
    Nguard=[0];
    Nint_ed=1;
end
if (nargin==18)
    pnseq_bin=[];
    Nguard=[0];
    Nint_ed=1;
end
if (nargin==19)
    Tguard=[0];
    Nint_ed=1;
end
if (nargin==20)
    Nint_ed=1;
end

if (Nu<1)
    Nu=1;
end

genseq=1;
if (isempty(pnseq))
    pnseq=[0];
end

if (istrellis(cc))
    encode=1;
    VD_depth=cc.numStates*5;
    Rcc=cc.numInputSymbols/cc.numOutputSymbols;
    mem_vd=log(cc.numStates)/log(2);
else
    Rcc=1;
    encode=0;
    mem_vd=0;
end
if (encode==0)
    soft_decode=0;
end

if (size(order,1)==2)
    order_ed=reducer0(order(2,:));
    order=reducer0(order(1,:));
else
    order_ed=[];
end
    

Noversample=round(Tc/dt);
Nqframe=Nlev*Noversample;
Lpn=size(pnseq,2);
Nqsymbol_useful=Nqframe*Lpn;
Nqsymbol=Nqframe*Lpn+Nguard*Noversample;

R=[1/(Nqsymbol*dt*1e-9)*Rcc,1/(Nqsymbol*dt*1e-9)];


if (size(pnseq,1)<Nu)
    warning('Error')
end

coherent_detection=1;
non_coherent_detection=1;
if (modulation==3)
    if (coherent_detection & non_coherent_detection)
        Ndetector=length(order)+length(order_ed)+1;
        Ndetector_coherent=length(order);
    end
    if (not(coherent_detection) & non_coherent_detection)
        Ndetector=length(order_ed)+1;
        Ndetector_coherent=0;
    end
else
    Ndetector=1;
    Ndetector_coherent=0;
end

if (soft_decode==2)
    Ndec_cc=2;
else
    Ndec_cc=1;
end

nsp=zeros(1,size(Esn0,2));

for isnr=1:length(Esn0)
    %
    fprintf('\nSimulating Es/N0 = %g ... \n',Esn0(isnr));
    nerr(1:(Ndetector+Ndetector_coherent)*Ndec_cc,isnr)=zeros((Ndetector+Ndetector_coherent)*Ndec_cc,1);
    nblk(1:(Ndetector+Ndetector_coherent)*Ndec_cc,isnr)=zeros((Ndetector+Ndetector_coherent)*Ndec_cc,1);
    nerr_ch(1:(Ndetector+Ndetector_coherent),isnr)=zeros((Ndetector+Ndetector_coherent),1);
    nblk_ch(1:(Ndetector+Ndetector_coherent),isnr)=zeros((Ndetector+Ndetector_coherent),1);
    %
    cont_channel=0;
    cont=0;
    while (nblk(1,isnr)<Nblkerrmax)&(nsp(1,isnr)<Ntot)
        %
        if (rem(nsp(1,isnr),10)==0)
            fprintf('%d/%d ',nsp(1,isnr),Ntot);
            cont=cont+1;
            if (cont==11)
                cont=0;
                fprintf('\n');
            end
        end
        nsp(1,isnr)=nsp(1,isnr)+1;
        %
        rxsig=[];
        for iu=1:Nu
            % channel
            if (channel==0)
                h=[1];
                t=[0];
                t0=0;
            else
                cont_channel=cont_channel+1;
                if (cont_channel==size(channel,1)+1)
                    cont_channel=1;
                end
                h=channel(cont_channel,:);
            end
            %        
            if (genseq)
                %
                bseq=[round(rand(1,Nbit))];
                if (iu==1)
                    bseqref=bseq;
                    href=h;
                end
                if (modulation==3)
                    bseq_add=[round(rand(1,Nbit))];
                    if (iu==1)
                        bseqref_add=bseq_add;
                    end
                end
                %
                % encoding
                if (encode)
                    [bseq_ch,fs_cc]=convenc([bseq,zeros(1,VD_depth)],cc);
                    % zero termination --> next version : tailbiting
                    bseq_ch=bseq_ch(1:length(bseq)/Rcc+mem_vd/Rcc); 
                    if (iu==1)
                        bseq_chref=bseq_ch;
                    end
                    if (modulation==3)
                        % additional channel
                        [bseq_add_ch,fs_add_cc]=convenc([bseq_add,zeros(1,VD_depth)],cc);
                        bseq_add_ch=bseq_add_ch(1:length(bseq_add)/Rcc+mem_vd/Rcc); 
                        if (iu==1)
                            bseq_add_chref=bseq_add_ch;
                        end
                    end
                else
                    bseq_ch=bseq;
                    if (iu==1)
                        bseq_chref=bseq_ch;
                    end
                    if (modulation==3)
                        bseq_add_ch=bseq_add;
                        if (iu==1)
                            bseq_add_chref=bseq_add_ch;
                        end
                    end
                end
                %
            end
            %
            if (modulation==3)
                recpulse=conv(pulse,h);
                Erecpulse=recpulse*recpulse'*dt;
                slevbin=pnseq_bin(iu,:);
                %
                rubin=zeros(1,Nqsymbol*(1+length(bseq_ch)));
                for im=1:length(bseq_ch)
                    if (bseq_ch(im)==0)
                        if (bseq_add_ch(im)==0)
                            rubin((im-1)*Nqsymbol+1+levels(1)*Noversample:im*Nqsymbol+levels(1)*Noversample)=-pnseq_bin(iu,:);
                        else
                            rubin((im-1)*Nqsymbol+1+levels(1)*Noversample:im*Nqsymbol+levels(1)*Noversample)=pnseq_bin(iu,:);
                        end
                    else
                        if (bseq_add_ch(im)==0)
                            rubin((im-1)*Nqsymbol+1+levels(2)*Noversample:im*Nqsymbol+levels(2)*Noversample)=-pnseq_bin(iu,:);
                        else
                            rubin((im-1)*Nqsymbol+1+levels(2)*Noversample:im*Nqsymbol+levels(2)*Noversample)=pnseq_bin(iu,:);
                        end
                    end
                end
                ru=conv(rubin,recpulse);           
            end
            %
            % 
            if (iu==1)
                Erecpulseref=Erecpulse;
                slevbinref=slevbin;
            end
            if (iu==1)
                P  = mean(abs(ru).^2); % average received power
                Ec = P*Nqsymbol;       % energy per single bit in the codeword Ec = P*(DF/PRF)*dSF*txOS
                Es = Ec/Rcc;           % energy per information symbol
                N0 = Es*(10^(-Esn0(isnr)/10));  % noise variance
                sigma = sqrt(N0/2);
                recpulseref=recpulse;
            end
            if (iu>1)&(asynch)
                offsetchip=floor(rand*(length(ru)-1));
                ru=[ru(offsetchip+1:end),ru(1:offsetchip)];
            end
            %
            if length(ru)>length(rxsig)
                rxsig=[rxsig,zeros(1,length(ru)-length(rxsig))];
            else
                ru=[ru,zeros(1,-length(ru)+length(rxsig))];
            end
            %
            rxsig=rxsig+ru;
        end
        %
        n=sigma*randn(1,length(rxsig))+i*sigma*randn(1,length(rxsig));
        rxsig=rxsig+n;
        %
        exist_err_ch=zeros(1,Ndetector+Ndetector_coherent);
        Psoft=[];
        in_bit=[];
        %
        % COHERENT / NON_COHERENT RECEIVER + CODE CC
        % RAKE / ED
        %
        if (coherent_detection)
            % Ideal Channel acquisition + timing estimation
            [flt]=rec_uwb_filter(0,pulse,order,[]);
            mfoutref=conv(recpulseref,flt);
            [m0,isampling]=max(abs(mfoutref));
            if (rem(isampling,Noversample)==0)
                begsampling=Noversample;
            else
                begsampling=rem(isampling,Noversample);
            end
            hrefds=mfoutref(begsampling:Noversample:end);
            [h0,ihr_sort]=sort(abs(hrefds));
            hrefds_sort=fliplr(hrefds(ihr_sort));
            ihr_sort=fliplr(ihr_sort);
            hrefds_sort=hrefds_sort(1:max(order));
            ihr_sort=ihr_sort(1:max(order));
            ihr_sort=ihr_sort-ihr_sort(1);
            min_ihr=min(ihr_sort);
            if (min_ihr<0)
                off_ihr=-min_ihr;
            else
                off_ihr=0;
            end
            lhest=length(hrefds_sort);
            m0ref=(hrefds*hrefds')*Lpn;
            %
            % Filtering
            mfout=conv(rxsig,flt);
            decd=mfout(isampling:Noversample:end);
            decd=[zeros(1,off_ihr),decd];
            decd_ri=decd;
        end % coherent_detection
        %
        if (non_coherent_detection)
            % Ideal Channel acquisition + timing estimation
            % Use the signal after the filter matche to the raised cosine
            % for decreasing noise effect before squaring
            edoutref=conv(abs(mfoutref).^2,ones(1,Noversample)); 
            [m0_ed,isampling_ed]=max(abs(edoutref));
            if (rem(isampling_ed,Noversample)==0)
                begsampling_ed=Noversample;
            else
                begsampling_ed=rem(isampling_ed,Noversample);
            end
            hrefds_ed=edoutref(begsampling_ed:Noversample:end);
            [hrefds_sort_ed,ihr_sort_ed]=sort(abs(hrefds_ed));
            hrefds_sort_ed=fliplr(hrefds_sort_ed);
            ihr_sort_ed=fliplr(ihr_sort_ed);
            if not(isempty(order_ed))
                hrefds_sort_ed=hrefds_sort_ed(1:max(order_ed));
                ihr_sort_ed=ihr_sort_ed(1:max(order_ed));
                ihr_sort_ed=ihr_sort_ed-ihr_sort_ed(1);
                lhest_ed=length(hrefds_sort_ed);
                min_ihr_ed=min(ihr_sort_ed);
                if (min_ihr_ed<0)
                    off_ihr_ed=-min_ihr_ed;
                else
                    off_ihr_ed=0;
                end
            end
            % Non selective non coherent integration
            ihr_ns_ed=[0:Nint_ed-1];
            %
            % Energy detection
            edout=conv(abs(mfout).^2,ones(1,Noversample)); 
            decd_ed=edout(isampling_ed:Noversample:end);
            decd_ed=[zeros(1,off_ihr_ed),decd_ed];
        end % non_coherent_detection
        %
        beg=0;
        for idet=1:length(bseq_ch)
            %
            cl=zeros(length(order),2);
            cl_ed=zeros(length(order)+1,2);
            cl_ri=zeros(1,2);
            for ih=1:length(pnseq)
                begcorr=beg+(ih-1)*Nlev+abs(pnseq(ih))+1;
                signcorr=sign(pnseq(ih));
                %
                if (modulation==3)
                    % PPM
                    if (coherent_detection)
                        % COHERENT RAKE RECEIVER - order
                        for iod=1:length(order)
                            clcorr=decd(begcorr+levels(1)+ihr_sort(1:order(iod))+off_ihr)*hrefds_sort(1:order(iod))'; % RAKE
                            cl(iod,1)=cl(iod,1)+signcorr*clcorr;
                            clcorr=decd(begcorr+levels(2)+ihr_sort(1:order(iod))+off_ihr)*hrefds_sort(1:order(iod))'; % RAKE
                            cl(iod,2)=cl(iod,2)+signcorr*clcorr;
                        end
                    end
                    %
                    if (non_coherent_detection)
                        % NON COHERENT RECEIVER - order_ed
                        for iod=1:length(order_ed)
                            clcorr=decd_ed(begcorr+levels(1)+ihr_sort_ed(1:order_ed(iod))+off_ihr_ed)*ones(1,order_ed(iod))'; % RAKE
                            cl_ed(iod,1)=cl_ed(iod,1)+clcorr;
                            clcorr=decd_ed(begcorr+levels(2)+ihr_sort_ed(1:order_ed(iod))+off_ihr_ed)*ones(1,order_ed(iod))'; % RAKE
                            cl_ed(iod,2)=cl_ed(iod,2)+clcorr;
                        end
                        % NON COHERENT RECEIVER - non selective
                        clcorr=decd_ed(begcorr+levels(1)+ihr_ns_ed+off_ihr_ed)*ones(1,length(ihr_ns_ed))'; % RAKE
                        cl_ed(length(order_ed)+1,1)=cl_ed(length(order_ed)+1,1)+clcorr;
                        clcorr=decd_ed(begcorr+levels(2)+ihr_ns_ed+off_ihr_ed)*ones(1,length(ihr_ns_ed))'; % RAKE
                        cl_ed(length(order_ed)+1,2)=cl_ed(length(order_ed)+1,2)+clcorr;
                    end
                end
                %
            end % ih
            % 
            if (modulation==3)
                % PPM
                % Demodulation order --> 
                % 1 - primary channel coherent,
                % 2 - primary channel non-coherent, 
                % 3 - additional channel coherent
                cont_dec=1;
                if (coherent_detection)
                    for iod=1:length(order)
                        % BPSK
                        cl_corr=real(cl(iod,:));
                        decbit(cont_dec)=(abs(cl_corr(1))<abs(cl_corr(2)));
                        if (decbit(cont_dec)==0)
                            decbit(cont_dec+Ndetector)=cl_corr(1)>0;
                        else
                            decbit(cont_dec+Ndetector)=cl_corr(2)>0;
                        end
                        if (soft_decode)
                            Psoft(cont_dec,idet)=abs(cl_corr(2))-abs(cl_corr(1)); % 1 > 0
                            if (decbit(cont_dec)==0)
                                Psoft(cont_dec+Ndetector,idet)=cl_corr(1); % 1 > 0
                            else
                                Psoft(cont_dec+Ndetector,idet)=cl_corr(2); % 1 > 0
                            end
                        end
                        cont_dec=cont_dec+1;
                    end
                end
                if (non_coherent_detection)
                    for iod=1:(length(order_ed)+1)
                        cl_corr=cl_ed(iod,:);
                        decbit(cont_dec)=(cl_corr(1)<cl_corr(2));
                        if (soft_decode)
                            Psoft(cont_dec,idet)=cl_corr(2)-cl_corr(1);
                        end
                        cont_dec=cont_dec+1;
                    end
                end
                %
            end
            %
            for iod=1:Ndetector
                if (bseq_chref(idet)~=decbit(iod))
                    nerr_ch(iod,isnr)=nerr_ch(iod,isnr)+1;   
                    exist_err_ch(iod)=1;
                end
                if (encode)
                    in_bit(iod,idet)=decbit(iod);
                end
            end
            if (coherent_detection)
                for iod=1:Ndetector_coherent
                    if (bseq_add_chref(idet)~=decbit(iod+Ndetector))
                        nerr_ch(Ndetector+iod,isnr)=nerr_ch(Ndetector+iod,isnr)+1;   
                        exist_err_ch(Ndetector+iod)=1;
                    end
                    if (encode)
                        in_bit(Ndetector+iod,idet)=decbit(iod+Ndetector);
                    end
                end
            end
            %
            beg=beg+Lpn*Nlev;
        end % idet
        %
        for iod=1:(Ndetector+Ndetector_coherent)
            if (exist_err_ch(iod))
                nblk_ch(iod,isnr)=nblk_ch(iod,isnr)+1;
            end
        end
        %
        % decoder
        if (encode)
            %
            parcont=1;
            % VITERBI PRIMARY CHANNEL
            for iod=1:(Ndetector)
                if (soft_decode==1)|(soft_decode==2)
                    Nlevels_dec=6;
                    minp=min(Psoft(iod,:));
                    maxp=max(Psoft(iod,:));
                    dp=(maxp-minp)/(2^Nlevels_dec-1);
                    in_soft=round((Psoft(iod,:)-minp)/dp);
                    dec_vit=vitdec([in_soft,zeros(1,round(VD_depth/Rcc))],cc,VD_depth,'cont','soft',Nlevels_dec);
                    dec_vit=dec_vit(VD_depth+1:VD_depth+length(bseqref));
                    nerr_corr=length(find(dec_vit-bseqref));
                    nerr(parcont,isnr)=nerr(parcont,isnr)+nerr_corr;
                    if (nerr_corr>0)
                        nblk(parcont,isnr)=nblk(parcont,isnr)+1;
                    end
                    parcont=parcont+1;
                end
                if (soft_decode==0)|(soft_decode==2)
                    dec_vit=vitdec([in_bit(iod,:),zeros(1,round(VD_depth/Rcc))],cc,VD_depth,'cont','hard');
                    dec_vit=dec_vit(VD_depth+1:VD_depth+length(bseqref));
                    nerr_corr=length(find(dec_vit-bseqref));
                    nerr(parcont,isnr)=nerr(parcont,isnr)+nerr_corr;
                    if (nerr_corr>0)
                        nblk(parcont,isnr)=nblk(parcont,isnr)+1;
                    end
                    parcont=parcont+1;
                end
            end 
            % VITERBI ADDITIONAL CHANNEL
            for iod=1:(Ndetector_coherent)
                if (soft_decode==1)|(soft_decode==2)
                    Nlevels_dec=6;
                    minp=min(Psoft(iod+Ndetector,:));
                    maxp=max(Psoft(iod+Ndetector,:));
                    dp=(maxp-minp)/(2^Nlevels_dec-1);
                    in_soft=round((Psoft(iod+Ndetector,:)-minp)/dp);
                    dec_vit=vitdec([in_soft,zeros(1,round(VD_depth/Rcc))],cc,VD_depth,'cont','soft',Nlevels_dec);
                    dec_vit=dec_vit(VD_depth+1:VD_depth+length(bseqref_add));
                    nerr_corr=length(find(dec_vit-bseqref_add));
                    nerr(parcont,isnr)=nerr(parcont,isnr)+nerr_corr;
                    if (nerr_corr>0)
                        nblk(parcont,isnr)=nblk(parcont,isnr)+1;
                    end
                    parcont=parcont+1;
                end
                if (soft_decode==0)|(soft_decode==2)
                    dec_vit=vitdec([in_bit(iod+Ndetector,:),zeros(1,round(VD_depth/Rcc))],cc,VD_depth,'cont','hard');
                    dec_vit=dec_vit(VD_depth+1:VD_depth+length(bseqref_add));
                    nerr_corr=length(find(dec_vit-bseqref_add));
                    nerr(parcont,isnr)=nerr(parcont,isnr)+nerr_corr;
                    if (nerr_corr>0)
                        nblk(parcont,isnr)=nblk(parcont,isnr)+1;
                    end
                    parcont=parcont+1;
                end
            end
            %
        end % if encode
        %
        %
    end % while --> nsp
    %
    %
end % for isnr --> SNR


